pub struct ThreadLocalMut<'a, T> { /* private fields */ }Expand description
Borrowed thread local variable.
This structure tracks borrow rules at runtime, it may be necessary to manually drop the object, if multiple rayon loops are involved.